
/**
 * A dropdown that can be queried and it's links selected via keyboard. Dropdown contents retrieved via AJAX.
 *
 * @module Controls
 * @since 4.2
 * @namespace AJS
 * @class QueryableDropdown
 * @requires AJS.$, AJS.SelectModel, AJS.Dropdown, AJS.List, AJS.Control
 */

 AJS.QueryableDropdown = AJS.Control.extend({

    /**
     *  A request will not be fired and suggestions will not reset if any of these keys are inputted.
     *
     * @proptery {Array} INVALID_KEYS
     */
    INVALID_KEYS: [
        jira.keyboard.SpecialKey.TAB,
        jira.keyboard.SpecialKey.ESC,
        jira.keyboard.SpecialKey.SHIFT,
        jira.keyboard.SpecialKey.RIGHT
    ],


    /**
     * Overrides default options with user options. Inserts an input field before dropdown.
     *
     * @constructor
     * @param {Object} options
     */
    init: function (options) {

        var instance = this;

        this._setOptions(options);

        // setting default properties
        this._queuedRequest = 0;
        this.suggestionsVisible = false;

        // if the minQueryLength property comes from the DOM we need to convert it to an int
        if (this.options.ajaxOptions.minQueryLength) {
            this.options.ajaxOptions.minQueryLength = parseInt(this.options.ajaxOptions.minQueryLength, 10);
        }

        this._createFurniture();

        this.dropdownController = AJS.InlineLayer.create({
            offsetTarget: this.$field,
            width: this.$field.innerWidth(),
            content: options.element
        });

        this.listController = new AJS.List({
            containerSelector: options.element,
            groupSelector: "ul.aui-list-section",
            itemSelector: "li",
            selectionHandler: function () {
                // prevent form field from being dirty
                instance.$field.val(AJS.params.dotLoading).css("color", "#999");
                instance.hideSuggestions();
                return true;
            }
        });

        
        this._assignEventsToFurniture();

        if(this.options.loadOnInit) {
            this.suggestionsDisabled = true;
            this._requestThenResetSuggestions();
        }
    },

    /**
     * Gets default options
     *
     * @method _getDefaultOptions
     * @private
     * @return {Object}
     */
    _getDefaultOptions: function () {
        return {
            id: "default",
            ajaxOptions: {
                data: { query: "" },
                dataType: "json",
                minQueryLength: 0
            },
            // keyInputPeriod: expected milliseconds between consecutive keystrokes
            // If this user types faster than this, no requests will be issued until they slow down.
            keyInputPeriod: 75
        };
    },

     /**
      * Gets ajaxOptions. We get them from the options object but copy it so that if we modify and properties it
      * won't change those on the options object through reference.
      *
      * @method getAjaxOptions
      * @return {Object}
      */
    getAjaxOptions: function () {
        this.options.ajaxOptions.data.query = AJS.$.trim(this.$field.val());
        return AJS.copyObject(this.options.ajaxOptions); // new object, no references
    },

    /**
     * Issue an ajax request.
     *
     * @method issueRequest
     */
    issueRequest: function() {

        var instance = this,
            ajaxOptions = this.getAjaxOptions();

        ajaxOptions.complete = function (xhr, textStatus, smartAjaxResult) {

            instance.outstandingRequest = null;

            if (!instance.$container.is(":visible")) {
                // If this field was removed from the page, the suggestions should not be displayed.
                return;
            }

            if (smartAjaxResult.successful) {
                instance._handleServerSuccess(smartAjaxResult);
            } else if (!smartAjaxResult.aborted) {
                instance.hideSuggestions();
                instance._handleServerError(smartAjaxResult);
            }
        };

        this.outstandingRequest = jira.ajax.makeRequest(ajaxOptions);

        AJS.$(this.outstandingRequest).throbber({target: this.$dropDownIcon});
    },

    /**
     * This is called when the AJAX request is finished, to ensure we are talking about the right request
     *
     * @method _handleServerSuccess
     * @param {Object} smartAjaxResult - the smart AJAX result object in play
     */
    _handleServerSuccess : function(smartAjaxResult) {
        if (this.options.loadOnInit || this.$field.val() == this.options.ajaxOptions.data.query) {
            this._handleSuggestionResponse(smartAjaxResult.data);
        }
    },

    /**
     * This is called when an error has happened during the AJAX request.  By default
     * we show an alert messages but other instances can do more specific error handling
     *
     * @method _handleServerError
     * @param {Object} smartAjaxResult  - the smart AJAX result object in play
     */
    _handleServerError : function(smartAjaxResult) {
        // we have an error of some sort so show an error message
        var errMsg = jira.ajax.buildSimpleErrorContent(smartAjaxResult);
        /* [alert] */
        alert(errMsg);
        /* [alert] end */
    },

    /**
     * Appends furniture around specified dropdown element. This includes:
     *
     * <ul>
     *  <li>Field - text field used fro querying</li>
     *  <li>Container - Wrapper used to contain all furniture</li>
     *  <li>Dropdown Icon - Button in right of field used to open dropdown via mouse</li>
     * </ul>
     *
     * @method _createFurniture
     * @private
     */
    _createFurniture: function () {
        this.$container = this._render("container").insertBefore(this.options.element);
        this.$field = this._render("field").appendTo(this.$container);
        this.$dropDownIcon = this._render("dropdownAndLoadingIcon", this._hasDropdownButton()).appendTo(this.$container);
        this.$suggestionsContainer = this._render("suggestionsContainer");
    },

    /**
     * Whether or not to display dropdown icon/button
     *
     * @method _hasDropdownButton
     * @protected
     * @return {Boolean}
     */
    _hasDropdownButton: function () {
        return this.options.showDropdownButton || this.options.ajaxOptions.minQueryLength === 0;
    },

    /**
     * Assigns events to DOM nodes
     *
     * @method _assignEventsToFurniture
     * @protected
     */
    _assignEventsToFurniture: function () {

        var instance = this;

        this._assignEvents("ignoreBlurElement", this.$container);
        this._assignEvents("ignoreBlurElement", this.dropdownController.$layer);

        if (this._hasDropdownButton()) {
            this._assignEvents("ignoreBlurElement", this.$dropDownIcon);
            this._assignEvents("dropdownAndLoadingIcon", this.$dropDownIcon);
        }

        // if this control is created as the result of a keydown event then we do no want to catch keyup or keypress for a moment
        setTimeout(function() {
            instance._assignEvents("field", instance.$field);
            instance._assignEvents("keys", instance.$field);
        }, 15);
    },

    /**
     * Returns true if there is cheched request, and the query setting is false (meaning you would not like to go back to
     * the server for every keystroke)
     *
     * @method _isValidRequest
     * @return {Boolean}
     */
    _useCachedRequest: function () {
        return !!(this.cachedList && !this.options.ajaxOptions.query);
    },

    /**
     * Returns true if the query setting is true (meaning you would like to go back to the server for every keystroke)
     * Or if there has not been any request yet, so we will just get the data once thanks sir.
     *
     * @method _isValidRequest
     * @return {Boolean}
     */
    _isValidRequest: function () {
        return this.options.ajaxOptions.query || (!this.cachedList && !this.outstandingRequest);
    },

    /**
     * Requests JSON formatted suggestions from specified resource. Resource is sepecified in the ajaxOptions object
     * passed to the constructed during initialization.
     *
     * If the query option of ajaxOptions is set to true, an ajax request will be made for every keypress. Otherwise
     * ajax request will be made only the first time the dropdown is shown.
     *
     * @method _requestThenResetSuggestions
     * @private
     * @param {Boolean} ignoreBuffer - flag to specify that gating by keyInputPeriod should be circumvented
     */
    _requestThenResetSuggestions: function (ignoreBuffer) {

        var instance = this;

        // More information needs to be communicated between data retriever (AJS.QueryableDropdown) and suggestion
        // handler (AJS.List). A refactor should address this problem, but in the meantime this hack seems to work
        // best. Fixes JRADEV-3583.
        this.listController._latestQuery = AJS.$.trim(this.$field.val());

        if (this._useCachedRequest()) {
            this._handleSuggestionResponse(this.cachedList);
        } else if (this._isValidRequest()) {

            if (ignoreBuffer && this.outstandingRequest) {
                this.outstandingRequest.abort();
                this.outstandingRequest = null;
            }

            clearTimeout(this._queuedRequest); // cancel any queued requests

            if (!this.outstandingRequest) {
                this.issueRequest();
            } else {
                this._queuedRequest = setTimeout(function () {
                    instance._requestThenResetSuggestions(ignoreBuffer);
                }, this.options.keyInputPeriod);
            }
        }
    },


    /**
     * Propagates list with suggestions, performs necessary formatting of data
     *
     * @method _handleSuggestionResponse
     * @private
     * @param {Object} data
     */
    _handleSuggestionResponse: function (data) {
        if (data !== this.cachedList) {
            if (this._formatResponse) {
                data = this._formatResponse(data);
            } else if (this.options.ajaxOptions.formatResponse) {
                data = this.options.ajaxOptions.formatResponse.call(this, data);
            }
        }
        this.cachedList = data;
        this._setSuggestions(this.cachedList);
    },

    /**
     *
     * Sets suggestions and shows them
     *
     * @method _setSuggestions
     * @param {Array} data
     */
    _setSuggestions: function (data) {

        //if the user hasn't entered anything yet and we load the data on init then don't show any suggestions!
        if(this.suggestionsDisabled) {
            this.suggestionsDisabled = false;
            return;
        }

        // JRADEV-2053: If the field is no longer focused (i.e. the user has already tabbed away) don't set
        // suggestions as it will bring focus back to this field.
        this.suggestionsVisible = true;

        if (data) {
            this.dropdownController.show();
            this.dropdownController.setWidth(this.$field.innerWidth());

            if (this.options.ajaxOptions.query) {
                this.listController.generateListFromJSON(data); // we are relying on the server to do the matching for us.
            } else {
                this.listController.generateListFromJSON(data, this.$field.val());
            }

            this.listController.enable();

        } else {
            this.hideSuggestions();
        }
    },

    _isValidInput: function (e) {
        return this.$field.is(":visible") && AJS.$.inArray(jira.keyboard.specialKeyEntered(e), this.INVALID_KEYS) === -1;
    },

    /**
     * Hides list if the is no value in input, otherwise shows and resets suggestions in dropdown
     *
     * @method _handleCharacterInput
     * @param {Boolean} ignoreBuffer - Do not wait for interval between key strokes for requests
     * @param {Boolean} ignoreQueryLength - Ignore min query length before request is issued
     * @private
     */
    _handleCharacterInput: function (ignoreBuffer, ignoreQueryLength) {
        this.suggestionsDisabled = false;
        if (ignoreQueryLength || AJS.$.trim(this.$field.val()).length >= this.options.ajaxOptions.minQueryLength) {
            if (this.options.ajaxOptions.url) {
                //as soon as a key is pressed remove the noloading class so that if we're still retrieving data
                //from the server the spinner is shown!
                this.$dropDownIcon.removeClass("noloading");
                this._requestThenResetSuggestions(ignoreBuffer);
            } else {
                this._setSuggestions(this.model.getUnSelectedDescriptors());
            }
        } else {
            this._setSuggestions();
        }
    },

    /**
     * Handles down key
     *
     * @method _handleDown
     * @param {Event} e
     */
    _handleDown: function(e) {
        if (!this.suggestionsVisible) {
            this._handleCharacterInput(true, true);
            e.stopPropagation(); // stopping propagation so listController doesn't move to next item
        }
    },

    /**
     * Cancels and pending or outstanding requests
     *
     * @method _rejectPendingRequests
     * @protected
     */
    _rejectPendingRequests: function () {
        if (this.outstandingRequest) {
            this.outstandingRequest.abort();
        }
        clearTimeout(this._queuedRequest);
    },

    /**
     * Hides suggestions
     *
     * @method hideSuggestions
     */
    hideSuggestions: function () {

        if (!this.suggestionsVisible) {
            return;
        }

        this._rejectPendingRequests();
        this.suggestionsVisible = false;
        this.$dropDownIcon.addClass("noloading");
        this.dropdownController.hide();
        this.listController.disable();
    },

    /**
     * Handles Escape key
     *
     * @method _handleEscape
     * @param {Event} e
     */
    _handleEscape: function (e) {
        if (this.suggestionsVisible) {
            e.stopPropagation();
            if (e.type === "keyup") {
                this.hideSuggestions();
            }
        }
    },

    keys: {
        down: function (e) {
            if (this._hasDropdownButton()) {
                this._handleDown(e);
            }
        },
        up: function (e) {
            e.preventDefault();
        },
        "return": function (e) {
            e.preventDefault();
        },
        onEdit: function (e, character) {
            var instance = this;
            // delay until character is insterted into field
            this.$field.one("keyup", function () {
                instance._handleCharacterInput();
            });
        }
    },

    _events: {

        dropdownAndLoadingIcon: {
            click: function (e) {
                this.$field.focus();
                if (this.suggestionsVisible) {
                    this.hideSuggestions();
                } else {
                    this._handleDown(e);
                }
                e.stopPropagation();
            }
        },

        field: {
            blur: function () {
                this.hideSuggestions();
            },
            click: function (e) {
                e.stopPropagation();
            },
            keyup: function (e) {
                if (e.keyCode === jira.keyboard.SpecialKey.toKeyCode(jira.keyboard.SpecialKey.ESC)) {
                    this._handleEscape(e);
                }
            },
            beforedeactivate: function (e) {
                if (this.ignoreBlurEvent) {
                    this.ignoreBlurEvent = false;
                    e.preventDefault();
                }
            }
        },

        keys: {
            "keydown keypress": function (e) {
                this._handleKeyEvent(e);
            }
        },

        ignoreBlurElement: {
            mousedown: function (e) {
                if (e.target !== this.$field[0]) {
                    this.ignoreBlurEvent = true;
                    e.preventDefault();
                }
            }
        }
    },

    _renders: {
        field: function () {
            return AJS.$("<input class='text' type='text' autocomplete='off' />");
        },
        container: function () {
            return AJS.$("<div class='queryable-select' id='" + this.options.id + "-queryable-container' />");
        },
        dropdownAndLoadingIcon: function (showDropdown) {
            var $element = AJS.$('<span class="icon noloading"><span>More</span></span>');
            if  (showDropdown) {
                $element.addClass("drop-menu");
            }
            return $element;
        },
        suggestionsContainer : function () {
            return AJS.$("<div class='aui-list' id='" + this.options.id + "' tabindex='-1'></div>");
        }
    }

});
