/**
 * Converts a &lt;fielset&gt; of paramaters into a component picker
 *
 * HTML:
 *
 * <fieldset id="labels" class="hidden labelpicker-params">
 *       <input type="hidden" title="id" value="labels" />
 *       <input title="issueId" type="hidden" value="10072" />
 *       <input title="issueKey" type="hidden" value="MKY-3" />
 *       <select multiple="multiple" name="whatever"></select>
 * </fieldset>
 *
 * JavaScript:
 *
 * jira.app.findLabelsFieldsetAndConvertToPicker(document.body, "#fixVersions")
 *
 * @method findLabelsFieldsetAndConvertToPicker
 * @param {String, HTMLElement, JQuery} context - context to perform the selector search. If none supplied will search entire DOM
 * @param {String} selector - jQuery selector matching either a parent of the &lt;select&gt; or the select itself.
 *
 * If none supplied will used default selector. Which is:
 *
 * <pre>
 *  div.aui-field-labelpicker, // aui forms
 *  td.aui-field-labelpicker, // convert to subtask and move
 *  tr.aui-field-labelpicker // bulk edit
 * </pre>
 *
 * @namespace jira.app
 */
jira.app.findLabelsFieldsetAndConvertToPicker = function () {

    function createPicker($fieldset) {

        var opts = AJS.parseOptionsFromFieldset($fieldset),
            $select = AJS.$('#' + opts.id),
            issueId = opts.issueId,
            data = {};

        if (/customfield_\d/.test($select.attr('id'))) {
            data.customFieldId = parseInt($select.attr('id').replace('customfield_', ''), 10);
        }

        new AJS.LabelPicker({
            element: $select,
            ajaxOptions: {
                url: contextPath + '/rest/api/1.0/labels' + (issueId ? '/' + issueId : '') + '/suggest',
                data: data
            }
        });
    }

    function locateFieldset(parent) {

        var $parent = AJS.$(parent),
            $fieldset;

        if ($parent.is(FIELDSET_SELECTOR)) {
            $fieldset = $parent;
        } else {
            $fieldset = $parent.find(FIELDSET_SELECTOR);
        }

        return $fieldset;
    }

    var DEFAULT_SELECTORS = [
            "div.aui-field-labelpicker", // aui forms
            "td.aui-field-labelpicker", // convert to subtask and move
            "tr.aui-field-labelpicker" // bulk edit
        ],

        FIELDSET_SELECTOR = "fieldset.labelpicker-params";

    return function (context, selector) {

        selector = selector || DEFAULT_SELECTORS.join(", ");

        AJS.$(selector, context).each(function () {
            var $fieldset = locateFieldset(this);

            if ($fieldset.length) {
                createPicker($fieldset);
            } else {
                /* [logging] */
                AJS.log("jira.app.findLabelsFieldsetAndConvertToPicker: Tried to create label picker using [" + selector + "] "
                        + "however selector does not match or could not find <fieldset>");
                /* [logging] end */
            }

        });
    };
    
}();
